Package: progeny
Title: Pathway RespOnsive GENes for activity inference from gene
        expression
Version: 1.34.0
Authors@R: c(
    person(given = "Michael",
           family = "Schubert",
           role = "aut"),
    person(given = "Alberto",
           family = "Valdeolivas",
           role = "ctb",
           email = "alvaldeolivas@gmail.com",
           comment = c(ORCID = "0000-0001-5482-9023")),
    person(given = "Christian H.",
           family = "Holland",
           role = "ctb",
           email = "cholland2408@gmail.com",
           comment = c(ORCID = "0000-0002-3060-5786")),
    person(given = "Igor",
           family = "Bulanov",
           role = "ctb"),
    person(given = "Aurélien",
           family = "Dugourd",
           role = c("cre","ctb"), 
           email = "aurelien.dugourd@bioquant.uni-heidelberg.de")
           )
Description: PROGENy is resource that leverages a large compendium of publicly available signaling perturbation experiments to yield a common core of pathway responsive genes for human and mouse. These, coupled with any statistical method, can be used to infer pathway activities from bulk or single-cell transcriptomics. 
URL: https://github.com/saezlab/progeny
BugReports: https://github.com/saezlab/progeny/issues
Depends: R (>= 3.6.0)
Imports: Biobase, stats, dplyr, tidyr, ggplot2, ggrepel, gridExtra,
        decoupleR, reshape2
biocViews: SystemsBiology, GeneExpression, FunctionalPrediction,
        GeneRegulation
License: Apache License (== 2.0) | file LICENSE
LazyData: true
LazyDataCompression: bzip2
Encoding: UTF-8
Suggests: airway, biomaRt, BiocFileCache, broom, Seurat,
        SingleCellExperiment, DESeq2, BiocStyle, knitr, readr, readxl,
        pheatmap, tibble, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/progeny
git_branch: RELEASE_3_23
git_last_commit: cca15a9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:58:25 UTC; biocbuild
Author: Michael Schubert [aut],
  Alberto Valdeolivas [ctb] (ORCID:
    <https://orcid.org/0000-0001-5482-9023>),
  Christian H. Holland [ctb] (ORCID:
    <https://orcid.org/0000-0002-3060-5786>),
  Igor Bulanov [ctb],
  Aurélien Dugourd [cre, ctb]
Maintainer: Aurélien Dugourd <aurelien.dugourd@bioquant.uni-heidelberg.de>

Package: progeny
Title: Pathway RespOnsive GENes for activity inference from gene
        expression
Version: 1.33.0
Authors@R: c(
    person(given = "Michael",
           family = "Schubert",
           role = "aut"),
    person(given = "Alberto",
           family = "Valdeolivas",
           role = "ctb",
           email = "alvaldeolivas@gmail.com",
           comment = c(ORCID = "0000-0001-5482-9023")),
    person(given = "Christian H.",
           family = "Holland",
           role = "ctb",
           email = "cholland2408@gmail.com",
           comment = c(ORCID = "0000-0002-3060-5786")),
    person(given = "Igor",
           family = "Bulanov",
           role = "ctb"),
    person(given = "Aurélien",
           family = "Dugourd",
           role = c("cre","ctb"), 
           email = "aurelien.dugourd@bioquant.uni-heidelberg.de")
           )
Description: PROGENy is resource that leverages a large compendium of publicly available signaling perturbation experiments to yield a common core of pathway responsive genes for human and mouse. These, coupled with any statistical method, can be used to infer pathway activities from bulk or single-cell transcriptomics. 
URL: https://github.com/saezlab/progeny
BugReports: https://github.com/saezlab/progeny/issues
Depends: R (>= 3.6.0)
Imports: Biobase, stats, dplyr, tidyr, ggplot2, ggrepel, gridExtra,
        decoupleR, reshape2
biocViews: SystemsBiology, GeneExpression, FunctionalPrediction,
        GeneRegulation
License: Apache License (== 2.0) | file LICENSE
LazyData: true
LazyDataCompression: bzip2
Encoding: UTF-8
Suggests: airway, biomaRt, BiocFileCache, broom, Seurat,
        SingleCellExperiment, DESeq2, BiocStyle, knitr, readr, readxl,
        pheatmap, tibble, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/progeny
git_branch: devel
git_last_commit: 397f983
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:28:38 UTC; biocbuild
Author: Michael Schubert [aut],
  Alberto Valdeolivas [ctb] (ORCID:
    <https://orcid.org/0000-0001-5482-9023>),
  Christian H. Holland [ctb] (ORCID:
    <https://orcid.org/0000-0002-3060-5786>),
  Igor Bulanov [ctb],
  Aurélien Dugourd [cre, ctb]
Maintainer: Aurélien Dugourd <aurelien.dugourd@bioquant.uni-heidelberg.de>

# Generated by roxygen2: do not edit by hand

S3method(progeny,ExpressionSet)
S3method(progeny,Seurat)
S3method(progeny,SingleCellExperiment)
S3method(progeny,default)
S3method(progeny,matrix)
export(getModel)
export(progeny)
export(progenyPerm)
export(progenyScatter)
export(saveProgenyPlots)
import(ggplot2)
import(ggrepel)
import(gridExtra)
importFrom(dplyr,group_by)
importFrom(dplyr,select)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(reshape2,melt)
importFrom(stats,complete.cases)
importFrom(stats,ecdf)
importFrom(stats,sd)
importFrom(tidyr,"%>%")
importFrom(tidyr,spread)

# PROGENy: Pathway RespOnsive GENes for activity inference <img src="man/figures/tool_logo.png" align="right" width="120" />

<!-- badges: start -->
![GitHub](https://img.shields.io/github/license/saezlab/progeny)
<!-- badges: end -->


## Overview

PROGENy is resource that leverages a large compendium of publicly available
signaling perturbation experiments to yield a common core of pathway responsive
genes for human and mouse. These, coupled with any statistical method, can be
used to infer pathway activities from bulk or single-cell transcriptomics. 

This is an R package for storing the pathway signatures. To infer pathway
activities, please check out
[decoupleR](https://doi.org/10.1093/bioadv/vbac016), available in
[R](https://saezlab.github.io/decoupleR/) or
[python](https://github.com/saezlab/decoupler-py).

## Installation

Progeny is available in
[Bioconductor](https://www.bioconductor.org/packages/release/bioc/html/progeny.html). 
In addition, one can install the development version from the Github repository: 

```r
## To install the package from Bioconductor
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("progeny")

## To install the development version from the Github repo:
devtools::install_github("saezlab/progeny")
```

## Updates

Since the original release, we have implemented some extensions in PROGENy:

1. **Extension to mouse**:
  Originally PROGENy was developed for the application to human data. 
  In a benchmark study we showed that PROGENy is also applicable to mouse data, 
  as described in 
  [Holland et al., 2019](https://doi.org/10.1016/j.bbagrm.2019.194431). 
  Accordingly, we included new parameters to run mouse version of PROGENy by 
  transforming the human genes to their mouse orthologs.
2. **Expanding Pathway Collection**:
  We expanded human and mouse PROGENy with the pathways Androgen, Estrogen and 
  WNT.
3. **Extension to single-cell RNA-seq data**:
  We showed that PROGENy can be applied to scRNA-seq data, as described in
  [Holland et al., 2020](https://doi.org/10.1186/s13059-020-1949-z)

## Citation

> Schubert M, Klinger B, Klünemann M, Sieber A, Uhlitz F, Sauer S, Garnett MJ, 
Blüthgen N, Saez-Rodriguez J. 2018. Perturbation-response genes reveal signaling
footprints in cancer gene expression. _Nature Communications_: 
[10.1038/s41467-017-02391-6](https://doi.org/10.1038/s41467-017-02391-6)

#'Compute progeny pathway scores and assesses significance based on permutations
#'
#'@param df A data.frame of n*m+1 dimension, where n is the number of omic
#'features to be considered and m is the number of samples/contrasts.
#'The first column should be the identifiers of the omic features. 
#'These identifiers must be coherent with the identifiers of the weight matrix.
#'@param weight_matrix A progeny coefficient matrix. the first column should be
#'the identifiers of the omic features and should be coherent with 
#'the identifiers provided in df.
#'@param k The number of permutations to be performed to generate
#'the null-distribution used to estimate the significance of progeny scores.  
#'The default value is 10000.
#'@param z_scores if true, the z-scores will be returned for 
#'the pathway activity estimations. Else, the function returns 
#'a normalized z-score value between -1 and 1.
#'@param get_nulldist if true, the null score distribution used for 
#'normalization will be returned along with the actual normalized score data 
#'frame.
#'@importFrom stats complete.cases sd ecdf
#'@export 
#'@return This function returns a list of two elements. The first element is 
#'a data frame of p*m+1 dimensions, where p is the number of progeny pathways,
#'and m is the number of samples/contrasts. Each cell represents the 
#'significance of a progeny pathway score for one sample/contrast. The 
#'significance ranges between -1 and 1. The significance is equal to x*2-1, x 
#'being the quantile of the progeny pathway score with respect to the null 
#'distribution. Thus, this significance can be interpreted as the equivalent of 
#'1-p.value two-sided test over an empirical distribution) with the sign 
#'indicating the direction of the regulation. The second element is the null 
#'distribution list (a null distribution is generated for each sample/contrast).
#'@examples
#' # use example gene expression matrix
#' gene_expression <- as.matrix(read.csv(system.file("extdata", 
#' "human_input.csv", package = "progeny"), row.names = 1))
#'
#' # calculate pathway activities
#' progeny(gene_expression, scale=TRUE, organism="Human", top=100, perm=10000)
#'@export
progenyPerm <- 
    function(df,weight_matrix,k = 10000, z_scores = TRUE,  get_nulldist = FALSE)
{
    resList <- list()
    if(get_nulldist) {
         nullDist_list <- list()
    }
  
    for(i in 2:length(df[1,])) {
        current_df <- df[,c(1,i)]
        current_df <- current_df[complete.cases(current_df),]
        t_values <- current_df[,2]
        current_weights <- weight_matrix
        names(current_df)[1] <- "ID"
        names(current_weights)[1] <- "ID"
    
        common_ids <- merge(current_df, current_weights, by = "ID")
        common_ids <- as.character(common_ids$ID)

        row.names(current_df) <- current_df$ID 
        current_df <- as.data.frame(current_df[common_ids,-1])
        row.names(current_weights) <- current_weights$ID
        current_weights <- as.data.frame(current_weights[common_ids,-1])
        current_mat <- as.matrix(current_df)
        current_weights <- t(current_weights)
    
        scores <- as.data.frame(current_weights %*% current_mat)
        null_dist_t <- replicate(k, sample(t_values,length(current_mat[,1]), 
            replace = FALSE))
        null_dist_scores <- current_weights %*% null_dist_t
    
        if(get_nulldist) {
            nullDist_list[[i-1]] <- null_dist_scores
        }
    
        if(z_scores) {
            scores$mean <- apply(null_dist_scores,1,mean)
            scores$sd <- apply(null_dist_scores,1,sd)
            resListCurrent <- (scores[,1]-scores[,2])/scores[,3]
            names(resListCurrent) <- names(weight_matrix[,-1])
            resList[[i-1]] <- resListCurrent
        } else {
            for(j in seq(1, length(weight_matrix[,-1]))) {
                ecdf_function <- ecdf(null_dist_scores[j,])
                scores[j,1] <- ecdf_function(scores[j,1])
        }
        score_probas <- scores*2-1
        resListCurrent <- score_probas[,1]
        names(resListCurrent) <- names(weight_matrix[,-1])
        resList[[i-1]] <- resListCurrent
        }
    }
    names(resList) <- colnames(df)[-1]
    resDf <- as.data.frame(resList)
    if(get_nulldist) {
        names(nullDist_list) <- names(df[,-1])
        return(list(resDf, nullDist_list))
    } else {
        return(t(resDf))
    }
}

#' Calculate PROGENy pathway scores from gene expression
#'
#' This function uses the linear model of pathway-responsive genes underlying
#' the PROGENy method. It transforms a gene expression matrix with HGNC/MGI gene
#' symbols in rows and sample names in columns into a pathway score matrix with
#' samples in rows and pathways in columns.
#'
#' The publication of the method is available at:
#' https://www.nature.com/articles/s41467-017-02391-6
#'
#' The supplied expression object has to contain HGNC/MGI symbols in rows. This
#' will, in most cases (and how we originally used it), be either normalized
#' gene expression of a microarray experiment or log-transformed (and
#' possible variance-stabilized) counts from an RNA-seq experiment.
#'
#' The human and mouse model matrices consists of 14 pathways and large set of 
#' genes with an associated p-value (p-value per gene and pathway) that accounts
#' for the importance of each gene on each pathway upon perturbation. 
#' Its coefficients are non-zero if the gene-pathway pair corresponds
#' to the top N genes (100 by default) that were up-regulated upon stimulation
#' of the pathway in a wide range of experiments. The value corresponds to the 
#' fitted z-score across experiments in our model fit. 
#' Only rows with at least one non-zero coefficient were included, as the rest 
#' is not used to infer pathway activity.
#'
#' @param expr   A gene expression object with HGNC/MGI symbols in rows and 
#'               samples in columns. In order to run PROGENy in single-cell 
#'               RNAseq data, it also accepts Seurat and SingleCellExperiment 
#'               object, taking the normalized counts for the computation.   
#' @param scale  A logical value indicating whether to scale the scores of each
#'               pathway to have a mean of zero and a standard deviation of one.
#'               It does not apply if we use permutations. 
#' @param organism The model organism - "Human" or "Mouse"

#' @param top    The top n genes for generating the model matrix according to
#'               significance (p-value)
#' @param perm   An interger detailing the number of permutations. No 
#'               permutations by default (1). When Permutations larger than 1,
#'               we compute progeny pathway scores and assesses their 
#'               significance using a gene sampling-based permutation strategy, 
#'               for a series of experimental samples/contrasts.
#' @param verbose    A logical value indicating whether to display a message  
#'                   about the number of genes used per pathway to compute 
#'                   progeny scores (i.e. number of genes present in the 
#'                   progeny model and in the expression dataset)
#' @param z_scores Only applies if the number of permutations is greater than 1. 
#'                 A logical value. TRUE: the z-scores will be returned for 
#'                 the pathway activity estimations. FALSE: the function returns 
#'                 a normalized z-score value between -1 and 1.  
#' @param get_nulldist Only applies if the number of permutations is greater 
#'                 than 1. A logical value. TRUE: the null distributions
#'                 generated to assess the signifance of the pathways scores 
#'                 is also returned. 
#' @param assay_name Only applies if the input is a Seurat object. It selects the
#'               name of the assay on which Progeny will be run. Default to: 
#'               RNA, i.e. normalized expression values.
#' @param return_assay Only applies if the input is a Seurat object. A logical 
#'               value indicating whether to return progeny results as a new 
#'               assay called Progeny in the Seurat object used as input. 
#'               Default to FALSE. 
#' @param ...    Additional arguments to be passed to the functions. 
#'               
#' @return       A matrix with samples in rows and pathways in columns. In case
#'               we run the method with permutations and the option get_nulldist
#'               to TRUE, we will get a list with two elements. The first 
#'               element is the matrix with the pathway activity as before. 
#'               The second elements is the null distributions that we generate
#'               to assess the signifance of the pathways scores. 
#' @export
#' @seealso \code{\link{progenyPerm}}
#' @examples
#' # use example gene expression matrix here, this is just for illustration
#' gene_expression <- as.matrix(read.csv(system.file("extdata", 
#' "human_input.csv", package = "progeny"), row.names = 1))
#'
#' # calculate pathway activities
#' pathways <- progeny(gene_expression, scale=TRUE, 
#'     organism="Human", top = 100, perm = 1)
progeny = function(expr, scale=TRUE, organism="Human", top = 100, perm = 1, 
    verbose = FALSE, z_scores = FALSE, get_nulldist = FALSE, assay_name = "RNA", 
    return_assay = FALSE, ...) {
    UseMethod("progeny")
}

#' @export
progeny.ExpressionSet = function(expr, scale=TRUE, organism="Human", top = 100,
    perm = 1, verbose = FALSE, z_scores = FALSE, get_nulldist = FALSE, ...) {
    
    progeny(Biobase::exprs(expr), scale=scale, organism=organism, top=top, 
        perm = perm, verbose = verbose,  z_scores = z_scores, 
        get_nulldist = get_nulldist)
}

#' @export
progeny.Seurat = function(expr, scale=TRUE, organism="Human", top = 100,
    perm = 1, verbose = FALSE, z_scores = FALSE, get_nulldist = FALSE, 
    assay_name = "RNA", return_assay = FALSE,...) {
    
    requireNamespace("Seurat")
    
    if (!is.logical(return_assay)){
        stop("return_assay should be a logical value")
    }
    
    if (scale & return_assay){
        warning("Scale and return_assay should not be both true. 
        Please use the function Seurat::ScaleData(object, assay = \"progeny\") 
        to scale PROGENy scores. Scale is set to FALSE")
        scale = FALSE
    }
    
    results <- progeny(as.matrix(Seurat::GetAssayData(expr, slot = "data", 
        assay = assay_name)), scale=scale, organism=organism, top=top, 
        perm = perm, verbose = verbose, z_scores = z_scores, 
        get_nulldist = get_nulldist, assay_name = assay_name, 
        return_assay = return_assay)
    
    if (return_assay){
        expr[['progeny']] = Seurat::CreateAssayObject(data = t(results))
        Seurat::Key(object = expr[['progeny']]) <- 'progeny_'
        return(expr)
    } else {
        return(results)
    }
    
}

#' @export
progeny.SingleCellExperiment =  function(expr, scale=FALSE, organism="Human", 
    top = 100, perm = 1, verbose = FALSE, z_scores = FALSE, 
    get_nulldist = FALSE, ...) {
    
    requireNamespace("SingleCellExperiment")

    progeny(as.matrix(SingleCellExperiment::normcounts(expr)), scale=scale, 
        organism=organism, top=top, perm = perm, verbose = verbose,  
        z_scores = z_scores, get_nulldist = get_nulldist)
}

#' @export
progeny.matrix = function(expr, scale=TRUE, organism="Human", top = 100, 
    perm = 1, verbose = FALSE,  z_scores = FALSE, get_nulldist = FALSE,...) {
  
    if (!is.logical(scale)){
        stop("scale should be a logical value")
    }
    
    if (!(is.numeric(perm)) || perm < 1){
        stop("perm should be an integer value")
    }
    
    if (!is.logical(verbose)){
        stop("verbose should be a logical value")
    }
    
    if (!is.logical(z_scores)){
        stop("z_scores should be a logical value")
    }
    
    if (!is.logical(get_nulldist)){
        stop("get_nulldist should be a logical value")
    }
    
    if (perm == 1 && (z_scores || get_nulldist)){
        if (verbose){
            message("z_scores and get_nulldist are only applicable when the
                number of permutations is larger than 1.")
        }
    }
    
    model <- getModel(organism, top=top)
    common_genes <- intersect(rownames(expr), rownames(model))

    if (verbose){
        number_genes <- apply(model, 2, function (x) {
            sum(rownames(model)[which (x != 0)] %in% unique(rownames(expr)))
        })
        message("Number of genes used per pathway to compute progeny scores:")
        message(paste(names(number_genes),": ", number_genes, " (", 
            (number_genes/top)*100,"%)",sep = "","\n"))
    }
    
    if (perm==1) {
        result <- t(expr[common_genes,,drop=FALSE]) %*% 
            as.matrix(model[common_genes,,drop=FALSE])
        
        if (scale && nrow(result) > 1) {
            rn <- rownames(result)
            result <- apply(result, 2, scale)
            rownames(result) <- rn
        }
    
    } else if (perm > 1) {
      expr <- data.frame(names = row.names(expr), row.names = NULL, expr)
      model <- data.frame(names = row.names(model), row.names = NULL, model)
      result <- progenyPerm(expr, model, k = perm, z_scores = z_scores, 
          get_nulldist = get_nulldist)
    }

    return(result)    
}

#' @export
progeny.default = function(expr, scale=TRUE, organism="Human", top = 100, 
    perm = 1, verbose = FALSE, z_scores = FALSE, get_nulldist = FALSE, ...) {
    stop("Do not know how to access the data matrix from class ", class(expr))
}

library(progeny)

#get input data
human_input <- as.matrix(read.csv(system.file("extdata", "human_input.csv", 
                                              package = "progeny"), 
                                  row.names = 1))
mouse_input <- as.matrix(read.csv(system.file("extdata", "mouse_input.csv", 
                                              package = "progeny"), 
                                  row.names = 1))

#get expected data
human_def_expected <- read.csv(system.file("extdata", "human_def_expected.csv", 
                                           package = "progeny"),
                               row.names = 1,
                               check.names = FALSE)
mouse_def_expected <- read.csv(system.file("extdata", "mouse_def_expected.csv", 
                                           package = "progeny"),
                               row.names = 1, 
                               check.names = FALSE)

#obtaining actual result
human_def_act <- as.data.frame(progeny(human_input, scale=TRUE, 
                                 organism = "Human", top = 10))
mouse_def_act <- as.data.frame(progeny(mouse_input, scale=TRUE, 
                                 organism = "Mouse", top = 10))

#testing
test_that("Comparison of the results", {
  expect_equal(human_def_act, human_def_expected)
  expect_equal(mouse_def_act, mouse_def_expected)
})

#' The full human linear model underlying PROGENy
#'
#' HGNC gene symbols in rows, pathways in columns. Pathway activity inference
#' works by matrix multiplication of gene expression with the model.
#'
#' @format The full human model contains 22479 genes, associated pathways,
#' weight and the p-value.         
#' \describe{
#'     \item{gene}{gene names in HGNC symbols}
#'     \item{pathway}{names of PROGENy pathways}
#'     \item{weight}{z-scores for a given gene}
#'     \item{p.value}{significance of gene in pathway}
#' }
#' @keywords datasets
#' @name model_human_full
#' @examples get("model_human_full", envir = .GlobalEnv)
#' @source \url{https://www.nature.com/articles/s41467-017-02391-6}
NULL

#' The full mouse linear model underlying PROGENy
#'
#' MGI gene symbols in rows, pathways in columns. Pathway activity inference
#' works by matrix multiplication of gene expression with the model.
#'
#' @format The full mouse model contains 17426 genes, associated pathways,
#' weight and the p-value.         
#' \describe{
#'     \item{gene}{gene names in HGNC symbols}
#'     \item{pathway}{names of PROGENy pathways}
#'     \item{weight}{z-scores for a given gene}
#'     \item{p.value}{significance of gene in a pathway}
#' }
#' @keywords datasets
#' @name model_mouse_full
#' @examples get("model_mouse_full", envir = .GlobalEnv)
#' @source \url{https://www.ncbi.nlm.nih.gov/pubmed/31525460}
NULL

#' The RNA data used in the progeny vignette
#'
#' List with three elements: the gene counts, the experimental design and 
#' the result of limma differential analysis
#'
#' @format List with three elements: the gene counts, the experimental design and 
#' the result of limma differential analysis        
#' \describe{
#'     \item{counts}{gene counts}
#'     \item{design}{experiemental design}
#'     \item{limma_ttop}{differential analysis result using limma}
#' }
#' @keywords datasets
#' @name vignette_data
#' @examples data("vignette_data")
#' @source \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE119931}
NULL

#' Calculate Progeny Scores with Permutations
#'
#'This function generate a series of scatter plot with marginal distribution
#'(in the form of an arrangeGrob object), for each progeny pathway and
#'sample/contrast. Each scatter plot has progeny weights as x-axis and the gene
#'level stat used to compute progeny score as the y-axis. The marginal
#'distribution of the gene level stats is displayed on the right of the plot
#'to give visual support of the significance of each gene contributing to
#'the progeny pathway score. The green and red colors represent the positive
#'and negative contribution of genes to the progeny pathway, respectively.
#'For each gene contribution, 4 cases are possible, as the combinations of
#'the sign of the gene level stat and the sign of the gene level weight.
#'Positive weight will lead to a positive(green)/negative(red) gene contribution
#'if the gene level stat is positive/negative. Negative weight will lead to
#'a negative(red)/positive(green) gene contribution if the gene level stat
#'is positive/negative.
#'
#'@param df an n*m data frame, where n is the number of omic features (genes). 
#'m isn't really important, as long as at least one column corresponds to a 
#'sample or contrast statistic. One of the columns should correspond to the gene 
#'symbols.
#'@param weight_matrix A progeny coefficient matrix. the first column should be
#'the identifiers of the omic features, and should be coherent with 
#'the identifiers provided in df.
#'@param dfID an integer corresponding to the column number of 
#'the gene identifiers of df.
#'@param weightID an integer corresponding to the column number of the gene 
#'identifiers of the weight matrix.
#'@param statName The name of the stat used, to be displayed on the plot
#'@param verbose  Logical indicating whether we want to have the messages 
#'indicating the different computed weights. 
#'@importFrom stats ecdf
#'@import ggplot2
#'@import ggrepel
#'@import gridExtra
#'@importFrom reshape2 melt
#'@return The function returns a list of list of arrangeGrob objects.
#'The first level list elements correspond to samples/contrasts. 
#'The second level correspond to pathways.
#'The plots can be saved in a pdf format using the saveProgenyPlots function.
#'@examples
#' # use example gene expression matrix
#' 
#' gene_expression <- read.csv(system.file("extdata", 
#' "human_input.csv", package = "progeny"))
#'
#' # getting a model matrix with 100 top significant genes and converting to df
#' weight_matrix <- getModel("Human", top=100)
#' weight_matrix <- data.frame(names = row.names(weight_matrix), 
#'   row.names = NULL, weight_matrix)
#' 
#' #use progenyScatter function
#' plots <- progenyScatter(gene_expression, weight_matrix)
#'@export
progenyScatter <- function(df,weight_matrix,dfID = 1, weightID = 1, 
    statName = "gene stats", verbose = FALSE) {
  
    weight <- color <- ID <- NULL    
    plot_list_contrasts <- list(0)
    for (i in 2:length(df[1,])) {
        plot_list_pathways <- list(0)
        for (j in 2:length(weight_matrix[1,])) {
            sub_df <- df[,c(dfID,i)]
            pathway_weights <- weight_matrix[,c(weightID,j)]
            names(sub_df) <- c("ID","stat")
            minstat <- min(sub_df$stat)
            maxstat <- max(sub_df$stat)
            
            histo <- ggplot(sub_df, aes(x = stat, fill = "blue")) + 
                geom_density() + coord_flip() + 
                scale_fill_manual(values = c("#00c5ff")) + 
                xlim(minstat, maxstat) + theme_minimal() + 
                theme(legend.position = "none", axis.text.x = element_blank(),
                axis.ticks.x = element_blank(), axis.title.y = element_blank(),
                axis.text.y = element_blank(), axis.ticks.y = element_blank(),
                panel.grid.major = element_blank(),
                panel.grid.minor = element_blank())
      
            names(pathway_weights) <- c("ID","weight")
            pathway_weights <- pathway_weights[pathway_weights$weight != 0,]
            percentile <- ecdf(sub_df$stat)
            sub_df <- merge(sub_df,pathway_weights,by = "ID")
            sub_df$color <- "3"
            sub_df[(sub_df$weight > 0 & sub_df$stat > 0),"color"] <- "1"
            sub_df[(sub_df$weight > 0 & sub_df$stat < 0),"color"] <- "2"
            sub_df[(sub_df$weight < 0 & sub_df$stat > 0),"color"] <- "2"
            sub_df[(sub_df$weight < 0 & sub_df$stat < 0),"color"] <- "1"
            sub_df[(percentile(sub_df$stat) < .95 & 
                percentile(sub_df$stat) > .05),1] <- NA
      
            if (verbose){
                message(paste("weights of ",names(weight_matrix)[j], sep = ""))
            }
          
            title <- paste("weights of ",names(weight_matrix)[j], sep = "")
      
            scatterplot <- ggplot(sub_df, aes(x = weight, y = stat, 
                color = color)) + geom_point() +
            scale_colour_manual(values = c("red","royalblue3","grey")) +
            geom_label_repel(aes(label = ID)) +
            ylim(minstat, maxstat) + theme_minimal() +
            theme(legend.position = "none") +
            geom_vline(xintercept = 0, linetype = 'dotted') +
            geom_hline(yintercept = 0, linetype = 'dotted') +
            labs(x = title, y = statName)
      
            lay <- t(as.matrix(c(1,1,1,1,2)))
            gg <- arrangeGrob(scatterplot, histo, nrow = 1, ncol = 2, 
                layout_matrix = lay)
            plot_list_pathways[[j-1]] <- gg
        }
        names(plot_list_pathways) <- names(weight_matrix[,-weightID])
        plot_list_contrasts[[i-1]] <- plot_list_pathways
    }
    return(plot_list_contrasts)
}

#'Function to save Progeny plots
#'
#'This function is designed to save the plots (in pdf format) of a nested 
#'(2 level) list of arrangeGrob objects, such as the one returned by 
#'the progenyScatter function.
#'
#'@param plots a list of list of arrangeGrob object (such as the one returned 
#'by the progenyScatter function.).The first level list elements correspond 
#'to samples/contrasts. The second level corresponds to pathways.
#'The plots can be saved in a pdf format using the saveProgenyPlots function.
#'@param contrast_names a vector of the same length as the first level of 
#'the plot list corresponding to the names of each sample/contrast
#'@param dirpath the path to the directory where the plots should be saved
#'@import ggplot2
#'@examples
#' #create plots using progneyScatter function
#' gene_expression <- read.csv(system.file("extdata", 
#' "human_input.csv", package = "progeny"))
#' 
#' # getting a weight_matrix
#' weight_matrix <- getModel("Human", top=100)
#' weight_matrix <- data.frame(names = row.names(weight_matrix), 
#'   row.names = NULL, weight_matrix) 
#' plots <- progenyScatter(gene_expression, weight_matrix)
#'
#' #create a list with contrast names
#' contrast_names <- names(gene_expression[2:ncol(gene_expression)])
#'
#' #assign a path to store your plots
#' dirpath <- "./progeny_plots/"
#' 
#' # save it
#' # saveProgenyPlots(plots, contrast_names, dirpath)
#' @return This function produces the pdf files of plots taken from the 
#' progenyScatter function
#'@export
saveProgenyPlots <- function(plots, contrast_names, dirpath) {
    
    i <- 1
    for (condition in plots) {
        dirname <- paste(dirpath,contrast_names[i], sep = "")
        dir.create(dirname, recursive = TRUE, showWarnings = FALSE)
        j <- 1
        for (pathway in condition) {
            filename <- paste(dirname,names(condition)[j],sep = "/")
            filename <- paste(filename,".pdf",sep = "")
            ggsave(filename, pathway,device = "pdf", dpi = 300)
            j <- j+1
        }
    i <- i+1
    }
}

#'Returns the Progeny Model
#'
#'This function is designed for getting a model matrix with top significant
#'genes for each pathway
#'
#'@param organism "Human" or "Mouse" taken from the main function's argument. 
#'Default to "Human"
#'@param top Desired top number of genes for each pathway according to their
#'significance(p.value). Default to 100
#'@param decoupleR if TRUE, the model matrix is goign to be generated with a format 
#'makes it directlz compatible with the decoupleR package
#'@examples #getting a model matrix according to the desired top n significant 
#'model <- getModel("Human", top=100)
#'@return This function returns model matrix according to the top n significant
#'@importFrom dplyr group_by top_n ungroup select 
#'@importFrom tidyr spread %>%
#'@export
getModel <- function(organism = "Human", top= 100, decoupleR = F) {
    
    pathway <- p.value <- weight <- NULL

    if (organism == "Human") {
        full_model <- progeny::model_human_full
    } else if (organism == "Mouse") {
        full_model <- progeny::model_mouse_full
    } else {
        stop("Wrong organism name. Please specify 'Human' or 'Mouse'.")
    }
    
    if (!(is.numeric(top)) || top < 1){
        stop("perm should be an integer value")
    }

    model <- full_model %>%
        dplyr::group_by(pathway) %>%
        dplyr::top_n(top, wt = -p.value) %>%
        dplyr::ungroup(pathway) %>%
        dplyr::select(-p.value) %>%
        tidyr::spread(pathway, weight, fill=0) %>%
        data.frame(row.names = 1, check.names = FALSE, stringsAsFactors = FALSE)
    
    if(decoupleR)
    {
      model$gene <- row.names(model)
      model <- melt(model)
      model <- model[model$value != 0,]
      model$mor <- sign(model$value)
      model$value <- abs(model$value)
      names(model) <- gsub("value","likelihood",names(model))
    }
  
  return(model)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----load, message=FALSE------------------------------------------------------
library(progeny)
library(ggplot2)
library(dplyr)

## ----model--------------------------------------------------------------------
model <- progeny::model_human_full
head(model)

## ----w_dist-------------------------------------------------------------------
# Get top 100 significant genes per pathway
model_100 <- model %>%
  group_by(pathway) %>%
  slice_min(order_by = p.value, n = 200)

# Plot
ggplot(data=model_100, aes(x=weight, color=pathway, fill=pathway)) +
  geom_density() +
  theme(text = element_text(size=12)) +
  facet_wrap(~ pathway, scales='free') +
  xlab('scores') +
  ylab('densities') +
  theme_bw() +
  theme(legend.position = "none")

## ----setup, echo=FALSE, results='hide', warning=FALSE, error=FALSE, message=FALSE, cache=FALSE----
library(knitr)
opts_chunk$set(
  cache = FALSE,
  echo = TRUE,
  warning = FALSE,
  error = FALSE,
  message = FALSE
)

## ----libraries, message=FALSE-------------------------------------------------
library(progeny)
library(tibble)
library(tidyr)
library(dplyr)
library(pheatmap)
library(readr)
library(ggplot2)

## ----loadInput, message=FALSE-------------------------------------------------
## We assign the normalised counts and the experimental design 
data("vignette_data")

Normalised_counts <- vignette_data$counts
Experimental_design <- vignette_data$design

## We read the results from the differential analysis. 
ttop_KOvsWT <- vignette_data$limma_ttop

## -----------------------------------------------------------------------------
Normalised_counts_matrix <- Normalised_counts %>% 
    dplyr::mutate_if(~ any(is.na(.x)),~ if_else(is.na(.x),0,.x)) %>% 
    tibble::column_to_rownames(var = "gene") %>% 
    as.matrix()

ttop_KOvsWT_matrix <- ttop_KOvsWT %>% 
    dplyr::select(ID, t) %>% 
    dplyr::filter(!is.na(t)) %>% 
    column_to_rownames(var = "ID") %>%
    as.matrix()

## ----ProgenyCounts, message=FALSE---------------------------------------------
PathwayActivity_counts <- progeny(Normalised_counts_matrix, scale=TRUE, 
    organism="Human", top = 100)
Activity_counts <- as.vector(PathwayActivity_counts)

## ----HeatmapProgeny, dpi=300--------------------------------------------------
paletteLength <- 100
myColor <- 
    colorRampPalette(c("darkblue", "whitesmoke","indianred"))(paletteLength)

progenyBreaks <- c(seq(min(Activity_counts), 0, 
    length.out=ceiling(paletteLength/2) + 1),
    seq(max(Activity_counts)/paletteLength, 
    max(Activity_counts), 
    length.out=floor(paletteLength/2)))

progeny_hmap <- pheatmap(t(PathwayActivity_counts),fontsize=14, 
    fontsize_row = 10, fontsize_col = 10, 
    color=myColor, breaks = progenyBreaks, 
    main = "PROGENy (100)", angle_col = 45,
    treeheight_col = 0,  border_color = NA)

## ----ProgenyStat, message=FALSE-----------------------------------------------
PathwayActivity_zscore <- progeny(ttop_KOvsWT_matrix, 
    scale=TRUE, organism="Human", top = 100, perm = 1000, z_scores = TRUE) %>%
    t()
colnames(PathwayActivity_zscore) <- "NES"

## ----BarplotProgeny, dpi=300--------------------------------------------------
PathwayActivity_zscore_df <- as.data.frame(PathwayActivity_zscore) %>% 
    rownames_to_column(var = "Pathway") %>%
    dplyr::arrange(NES) %>%
    dplyr::mutate(Pathway = factor(Pathway))

ggplot(PathwayActivity_zscore_df,aes(x = reorder(Pathway, NES), y = NES)) + 
    geom_bar(aes(fill = NES), stat = "identity") +
    scale_fill_gradient2(low = "darkblue", high = "indianred", 
        mid = "whitesmoke", midpoint = 0) + 
    theme_minimal() +
    theme(axis.title = element_text(face = "bold", size = 12),
        axis.text.x = 
            element_text(angle = 45, hjust = 1, size =10, face= "bold"),
        axis.text.y = element_text(size =10, face= "bold"),
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank()) +
    xlab("Pathways")

## ----ProgenySccater_1, message=FALSE, warning=FALSE---------------------------
prog_matrix <- getModel("Human", top=100) %>% 
    as.data.frame()  %>%
    tibble::rownames_to_column("GeneID")

ttop_KOvsWT_df <- ttop_KOvsWT_matrix %>% 
    as.data.frame() %>% 
    tibble::rownames_to_column("GeneID")

scat_plots <- progeny::progenyScatter(df = ttop_KOvsWT_df, 
    weight_matrix = prog_matrix, 
    statName = "t_values", verbose = FALSE)

## ----ProgenySccater_2, dpi=300------------------------------------------------
plot(scat_plots[[1]]$`MAPK`) 

## ----ProgenyCARNIVAL----------------------------------------------------------
PathwayActivity_CARNIVALinput <- progeny(ttop_KOvsWT_matrix, 
    scale=TRUE, organism="Human", top = 100, perm = 10000, z_scores = FALSE) %>%
    t () %>% 
    as.data.frame() %>% 
    tibble::rownames_to_column(var = "Pathway") 
colnames(PathwayActivity_CARNIVALinput)[2] <- "score"

## ----sessionInfo, echo=FALSE, eval=TRUE---------------------------------------
sessionInfo()

## ---- eval=FALSE--------------------------------------------------------------
#  if (!requireNamespace("BiocManager", quietly = TRUE))
#      install.packages("BiocManager")
#  
#  BiocManager::install("progeny")
#  
#  ## To install the new version until it is submitted to Bioconductor use:
#  devtools::install_github("saezlab/progeny")

## ---- message=FALSE-----------------------------------------------------------
library(progeny)
library(dplyr)
library(Seurat)
library(ggplot2)
library(tidyr)
library(readr)
library(pheatmap)
library(tibble)

## ---- eval=FALSE--------------------------------------------------------------
#  ## Load the PBMC dataset
#  pbmc.data <-
#      Read10X(data.dir = "../data/pbmc3k/filtered_gene_bc_matrices/hg19/")
#  ## Initialize the Seurat object with the raw (non-normalized data).
#  pbmc <-
#      CreateSeuratObject(counts = pbmc.data, project = "pbmc3k", min.cells = 3,
#      min.features = 200)

## ---- eval=TRUE , include=FALSE-----------------------------------------------
pbmc <- 
    readRDS(url("https://github.com/alberto-valdeolivas/ProgenyVignette/raw/master/SeuratObject.rds"))

## ---- message=FALSE-----------------------------------------------------------
## Identification of mithocondrial genes
pbmc[["percent.mt"]] <- PercentageFeatureSet(pbmc, pattern = "^MT-")

## Filtering cells following standard QC criteria.
pbmc <- subset(pbmc, subset = nFeature_RNA > 200 & nFeature_RNA < 2500 & 
    percent.mt < 5)

## Normalizing the data
pbmc <- NormalizeData(pbmc, normalization.method = "LogNormalize", 
    scale.factor = 10000)

pbmc <- NormalizeData(pbmc)

## Identify the 2000 most highly variable genes
pbmc <- FindVariableFeatures(pbmc, selection.method = "vst", nfeatures = 2000)

## In addition we scale the data
all.genes <- rownames(pbmc)
pbmc <- ScaleData(pbmc, features = all.genes)

## ---- message=FALSE, warning=FALSE--------------------------------------------
pbmc <- 
    RunPCA(pbmc, features = VariableFeatures(object = pbmc), verbose = FALSE)
pbmc <- FindNeighbors(pbmc, dims = 1:10, verbose = FALSE)
pbmc <- FindClusters(pbmc, resolution = 0.5, verbose = FALSE)
pbmc <- RunUMAP(pbmc, dims = 1:10,  umap.method = 'uwot', metric='cosine')

## -----------------------------------------------------------------------------
DimPlot(pbmc, reduction = "umap")

## ---- message = FALSE---------------------------------------------------------
## Finding differentially expressed features (cluster biomarkers)
pbmc.markers <- FindAllMarkers(pbmc, only.pos = TRUE, min.pct = 0.25, 
    logfc.threshold = 0.25, verbose = FALSE)

## Assigning cell type identity to clusters
new.cluster.ids <- c("Naive CD4 T", "Memory CD4 T", "CD14+ Mono", "B", "CD8 T", 
    "FCGR3A+ Mono", "NK", "DC", "Platelet")
names(new.cluster.ids) <- levels(pbmc)
pbmc <- RenameIdents(pbmc, new.cluster.ids)

## We create a data frame with the specification of the cells that belong to 
## each cluster to match with the Progeny scores.
CellsClusters <- data.frame(Cell = names(Idents(pbmc)), 
    CellType = as.character(Idents(pbmc)),
    stringsAsFactors = FALSE)

## ---- message = FALSE, warning = FALSE----------------------------------------
DimPlot(pbmc, reduction = "umap", label = TRUE, pt.size = 0.5) + NoLegend()

## ---- message = FALSE---------------------------------------------------------
## We compute the Progeny activity scores and add them to our Seurat object
## as a new assay called Progeny. 
pbmc <- progeny(pbmc, scale=FALSE, organism="Human", top=500, perm=1, 
    return_assay = TRUE)

## We can now directly apply Seurat functions in our Progeny scores. 
## For instance, we scale the pathway activity scores. 
pbmc <- Seurat::ScaleData(pbmc, assay = "progeny") 

## We transform Progeny scores into a data frame to better handling the results
progeny_scores_df <- 
    as.data.frame(t(GetAssayData(pbmc, slot = "scale.data", 
        assay = "progeny"))) %>%
    rownames_to_column("Cell") %>%
    gather(Pathway, Activity, -Cell) 

## We match Progeny scores with the cell clusters.
progeny_scores_df <- inner_join(progeny_scores_df, CellsClusters)

## We summarize the Progeny scores by cellpopulation
summarized_progeny_scores <- progeny_scores_df %>% 
    group_by(Pathway, CellType) %>%
    summarise(avg = mean(Activity), std = sd(Activity))

## ---- message=FALSE-----------------------------------------------------------
## We prepare the data for the plot
summarized_progeny_scores_df <- summarized_progeny_scores %>%
    dplyr::select(-std) %>%   
    spread(Pathway, avg) %>%
    data.frame(row.names = 1, check.names = FALSE, stringsAsFactors = FALSE) 

## -----------------------------------------------------------------------------
paletteLength = 100
myColor = colorRampPalette(c("Darkblue", "white","red"))(paletteLength)

progenyBreaks = c(seq(min(summarized_progeny_scores_df), 0, 
                      length.out=ceiling(paletteLength/2) + 1),
                  seq(max(summarized_progeny_scores_df)/paletteLength, 
                      max(summarized_progeny_scores_df), 
                      length.out=floor(paletteLength/2)))
progeny_hmap = pheatmap(t(summarized_progeny_scores_df[,-1]),fontsize=14, 
                        fontsize_row = 10, 
                        color=myColor, breaks = progenyBreaks, 
                        main = "PROGENy (500)", angle_col = 45,
                        treeheight_col = 0,  border_color = NA)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

---
title: "PROGENy pathway signatures: Application to Bulk transcriptomics"
author: "Alberto Valdeolivas"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{PROGENy pathway signatures: Application to Bulk transcriptomics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, echo=FALSE, results='hide', warning=FALSE, error=FALSE, message=FALSE, cache=FALSE}
library(knitr)
opts_chunk$set(
  cache = FALSE,
  echo = TRUE,
  warning = FALSE,
  error = FALSE,
  message = FALSE
)
```

## Introduction

This is the third part in a series of transcriptomics tutorials. We present here how to estimate pathway activity from transcriptomics 
data using **PROGENy**. 

Conventional pathway analysis methods rely on the gene expression of the pathway 
members. However, this approach overlooks the effect of post-translational 
modifications and only captures very specific experimental conditions. 
To overcome these limitations, **PROGENy** (Pathway RespOnsive GENes) estimates 
the activity of relevant signaling pathways based on consensus gene signatures 
obtained from perturbation experiments [(Schubert et al. 2018)](https://www.nature.com/articles/s41467-017-02391-6)

**PROGENy** initially contained 11 pathways and was developed for the 
application to human transcriptomics data. It has been recently shown that 
**PROGENy** is also applicable to mouse data and it has been expanded to 
14 pathways [(Holland et al., 2019)](https://doi.org/10.1016/j.bbagrm.2019.194431). 
In addition, **PROGENy** can be applied to scRNA-seq data, as described in 
[(Holland et al., 2020)](https://doi.org/10.1186/s13059-020-1949-z)

**PROGENy** is available as a [Bioconductor package](http://bioconductor.org/packages/release/bioc/html/progeny.html). 
For additional information about the **PROGENy** method, visit its website: 

https://saezlab.github.io/progeny/

## Getting Started

We first load the required libraries. 

```{r libraries, message=FALSE}
library(progeny)
library(tibble)
library(tidyr)
library(dplyr)
library(pheatmap)
library(readr)
library(ggplot2)
```

### Import the raw count dataframe
The data can be downloaded from: https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE119931
The file name is: GSE119931_PANC1.FOXA2KO.genes.counts.txt.gz 

```{r loadInput, message=FALSE}
## We assign the normalised counts and the experimental design 
data("vignette_data")

Normalised_counts <- vignette_data$counts
Experimental_design <- vignette_data$design

## We read the results from the differential analysis. 
ttop_KOvsWT <- vignette_data$limma_ttop
```

We have to slightly modify the format of the input files to make it suitable
for running Progeny. 

```{r}
Normalised_counts_matrix <- Normalised_counts %>% 
    dplyr::mutate_if(~ any(is.na(.x)),~ if_else(is.na(.x),0,.x)) %>% 
    tibble::column_to_rownames(var = "gene") %>% 
    as.matrix()

ttop_KOvsWT_matrix <- ttop_KOvsWT %>% 
    dplyr::select(ID, t) %>% 
    dplyr::filter(!is.na(t)) %>% 
    column_to_rownames(var = "ID") %>%
    as.matrix()
```


## Pathway activity with Progeny

We first compute **Progeny** scores for every sample (with the replicates) 
using the normalised counts. It is worth noting that we are going to use the 100
most responsive genes per pathway. This number can be increased depending on 
the coverage of your experiments. For instance, the number of quantified genes
for single-cell RNA-seq is smaller than for Bulk RNA-seq or microarray. In those
cases, we suggest to increase the number of responsive genes to 200-500. 

```{r ProgenyCounts, message=FALSE}
PathwayActivity_counts <- progeny(Normalised_counts_matrix, scale=TRUE, 
    organism="Human", top = 100)
Activity_counts <- as.vector(PathwayActivity_counts)
```

We present the results in a heatmap:

```{r HeatmapProgeny, dpi=300}
paletteLength <- 100
myColor <- 
    colorRampPalette(c("darkblue", "whitesmoke","indianred"))(paletteLength)

progenyBreaks <- c(seq(min(Activity_counts), 0, 
    length.out=ceiling(paletteLength/2) + 1),
    seq(max(Activity_counts)/paletteLength, 
    max(Activity_counts), 
    length.out=floor(paletteLength/2)))

progeny_hmap <- pheatmap(t(PathwayActivity_counts),fontsize=14, 
    fontsize_row = 10, fontsize_col = 10, 
    color=myColor, breaks = progenyBreaks, 
    main = "PROGENy (100)", angle_col = 45,
    treeheight_col = 0,  border_color = NA)
```


Now, we run an enrichment analysis using a competitive permutation approach to 
assess the significance of the pathway activity. We end up with Normalised Enrichment Scores (NES) for each pathway.

```{r ProgenyStat, message=FALSE}
PathwayActivity_zscore <- progeny(ttop_KOvsWT_matrix, 
    scale=TRUE, organism="Human", top = 100, perm = 1000, z_scores = TRUE) %>%
    t()
colnames(PathwayActivity_zscore) <- "NES"
```

```{r BarplotProgeny, dpi=300}
PathwayActivity_zscore_df <- as.data.frame(PathwayActivity_zscore) %>% 
    rownames_to_column(var = "Pathway") %>%
    dplyr::arrange(NES) %>%
    dplyr::mutate(Pathway = factor(Pathway))

ggplot(PathwayActivity_zscore_df,aes(x = reorder(Pathway, NES), y = NES)) + 
    geom_bar(aes(fill = NES), stat = "identity") +
    scale_fill_gradient2(low = "darkblue", high = "indianred", 
        mid = "whitesmoke", midpoint = 0) + 
    theme_minimal() +
    theme(axis.title = element_text(face = "bold", size = 12),
        axis.text.x = 
            element_text(angle = 45, hjust = 1, size =10, face= "bold"),
        axis.text.y = element_text(size =10, face= "bold"),
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank()) +
    xlab("Pathways")
```

The MAPK pathway is the most active pathway upon the perturbation that we 
are studying (KO of the FOXA2 gene versus the wild-type). We can therefore 
visualise the MAPK most responsive genes (progeny weights) along with their 
t_values to interpret the results. In the scatterplot, we can see the genes
that are contributing the most to the pathway enrichment. 

```{r ProgenySccater_1, message=FALSE, warning=FALSE}
prog_matrix <- getModel("Human", top=100) %>% 
    as.data.frame()  %>%
    tibble::rownames_to_column("GeneID")

ttop_KOvsWT_df <- ttop_KOvsWT_matrix %>% 
    as.data.frame() %>% 
    tibble::rownames_to_column("GeneID")

scat_plots <- progeny::progenyScatter(df = ttop_KOvsWT_df, 
    weight_matrix = prog_matrix, 
    statName = "t_values", verbose = FALSE)
```

```{r ProgenySccater_2, dpi=300}
plot(scat_plots[[1]]$`MAPK`) 
```

Progeny results can be used as an optional input for **CARNIVAL**. CARNIVAL
sets weights based on **PROGENy** scores in each pathway-related node in order
to find more relevant solutions. We therefore run **PROGENy** again with 
slightly different parameters, setting `z_scores = FALSE` so that **PROGENy** returns pathway activity values between 1 and -1, rather than converting to Z-Scores.

```{r ProgenyCARNIVAL}
PathwayActivity_CARNIVALinput <- progeny(ttop_KOvsWT_matrix, 
    scale=TRUE, organism="Human", top = 100, perm = 10000, z_scores = FALSE) %>%
    t () %>% 
    as.data.frame() %>% 
    tibble::rownames_to_column(var = "Pathway") 
colnames(PathwayActivity_CARNIVALinput)[2] <- "score"
```

PathwayActivity_CARNIVALinput can then be used as input for CARNIVAL.
## References

> Schubert M, Klinger B, Klünemann M, Sieber A, Uhlitz F, Sauer S, Garnett MJ, Blüthgen N, Saez-Rodriguez J. “Perturbation-response genes reveal signaling footprints in cancer gene expression.” _Nature Communications_: [10.1038/s41467-017-02391-6](https://doi.org/10.1038/s41467-017-02391-6)

> Holland CH, Szalai B, Saez-Rodriguez J. "Transfer of regulatory knowledge from human to mouse for functional genomics analysis." _Biochimica et Biophysica Acta (BBA) - Gene Regulatory Mechanisms._ 2019. DOI: [10.1016/j.bbagrm.2019.194431](https://doi.org/10.1016/j.bbagrm.2019.194431).

> Holland CH, Tanevski J, Perales-Patón J, Gleixner J, Kumar MP, Mereu E, Joughin BA, Stegle O, Lauffenburger DA, Heyn H, Szalai B, Saez-Rodriguez, J. "Robustness and applicability of transcription factor and pathway analysis tools on single-cell RNA-seq data." _Genome Biology._ 2020. DOI: [10.1186/s13059-020-1949-z](https://doi.org/10.1186/s13059-020-1949-z).

## Session Info Details

```{r sessionInfo, echo=FALSE, eval=TRUE}
sessionInfo()
```

---
title: "PROGENy pathway signatures"
author:
  - name: Pau Badia-i-Mompel
    affiliation:
    - Heidelberg Universiy
output: 
  BiocStyle::html_document:
    self_contained: true
    toc: true
    toc_float: true
    toc_depth: 3
    code_folding: show
package: "`r pkg_ver('progeny')`"
vignette: >
  %\VignetteIndexEntry{PROGENy pathway signatures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```


# Introduction

PROGENy is resource that leverages a large compendium of publicly available
signaling perturbation experiments to yield a common core of pathway responsive
genes. For each pathway, a collection of genes are available along their
contribution and significance to it.

Inside PROGENy, one can find gene signatures for 14 different pathways:

- **Androgen**: involved in the growth and development of the male reproductive organs.
- **EGFR**: regulates growth, survival, migration, apoptosis, proliferation, and differentiation in mammalian cells
- **Estrogen**: promotes the growth and development of the female reproductive organs.
- **Hypoxia**: promotes angiogenesis and metabolic reprogramming when O2 levels are low.
- **JAK-STAT**: involved in immunity, cell division, cell death, and tumor formation.
- **MAPK**: integrates external signals and promotes cell growth and proliferation.
- **NFkB**: regulates immune response, cytokine production and cell survival.
- **p53**: regulates cell cycle, apoptosis, DNA repair and tumor suppression.
- **PI3K**: promotes growth and proliferation.
- **TGFb**: involved in development, homeostasis, and repair of most tissues.
- **TNFa**: mediates haematopoiesis, immune surveillance, tumour regression and protection from infection.
- **Trail**: induces apoptosis.
- **VEGF**: mediates angiogenesis, vascular permeability, and cell migration.
- **WNT**: regulates organ morphogenesis during development and tissue repair.

# Activity estimation
These signatures, coupled with any statistical method,
can be used to infer pathway activities from bulk or single-cell
transcriptomics. 
In this vignette we just show how to access these signatures and some of their
properties. To infer pathway activities, please check out
[decoupleR](https://doi.org/10.1093/bioadv/vbac016), available in
[R](https://saezlab.github.io/decoupleR/) or
[python](https://github.com/saezlab/decoupler-py).

# Load
First we load the necessary packages:
```{r load, message=FALSE}
library(progeny)
library(ggplot2)
library(dplyr)
```

Here is how to retrieve all genes from each pathway in human:
```{r model}
model <- progeny::model_human_full
head(model)
```

Here we can observe how some genes behave for some pathways. For example,
The gene CBX6 has a negative response to EGFR, meaning that when there is EGFR
signaling this gene is down-regulated. On the other hand, the gene RFC2 has a
positive weight for EGFR, meaning that when there is EGFR signaling it becomes
up-regulated. We can also see the significance of each gene to each pathway. To
better estimate pathway activities, we recommend to select the top 100
significant genes for each pathway or filter by significance.

# Exploration
We can visualize the distribution of weights for the top 100 genes per pathway:

```{r w_dist}
# Get top 100 significant genes per pathway
model_100 <- model %>%
  group_by(pathway) %>%
  slice_min(order_by = p.value, n = 200)

# Plot
ggplot(data=model_100, aes(x=weight, color=pathway, fill=pathway)) +
  geom_density() +
  theme(text = element_text(size=12)) +
  facet_wrap(~ pathway, scales='free') +
  xlab('scores') +
  ylab('densities') +
  theme_bw() +
  theme(legend.position = "none")

```

Each pathway show a different distribution of weights. Some up-regulate more genes
than down-regulate, like NFkB or TNFa, while others do the opposite, like PI3K
or VEGF.

---
title: "Applying PROGENy on single-cell RNA-seq data"
author:
- name: Alberto Valdeolivas
  affiliation: Institute for Computational Biomedicine, Heidelberg University
  email: alberto.valdeolivas@bioquant.uni-heidelberg.de
- name: Igor Bulanov
  affiliation: Institute for Computational Biomedicine, Heidelberg University
- name: Christian Holland 
  affiliation: Institute for Computational Biomedicine, Heidelberg University
- name: Julio Saez-Rodriguez 
  affiliation: Institute for Computational Biomedicine, Heidelberg University 
   
package: progeny
output:
  BiocStyle::html_document
bibliography: Mybib2.bib
abstract: |
  This vignette describes how to infer the activity of 14 relevant signaling 
  pathways by running PROGENy on single-cell RNA-seq data. 
license: GNU-GLPv3, please check http://www.gnu.org/licenses/
vignette: |
  %\VignetteIndexEntry{Applying PROGENy on single-cell RNA-seq data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

Conventional pathway analysis methods rely on the gene expression of the pathway 
members. However, this approach overlook the effect of post-translational 
modifications and only captures very specific experimental conditions. 
To overcome these limitations, **PROGENy** (Pathway RespOnsive GENes) estimates 
the activity of relevant signaling pathways based on consensus gene signatures 
obtained from perturbation experiments, in other words, the footprint of the 
pathway on gene expression [@Schubert2018 ,@dugourd2019].

**PROGENy** initially contained 11 pathways and was developed for the 
application to human transcriptomics data. It has been recently shown that
**PROGENy** is also applicable to mouse data [@HOLLAND2019194431] and to single 
cell RNAseq data [@Holland2019]. In addition, they expanded human and mouse 
**PROGENy** to 14 pathways.

This vignette shows an example on how to apply **PROGENy** in a well known 
single-cell dataset. To analyse the data, we followed the standard procedures of
the `r CRANpkg("Seurat")` toolkit for single cell genomics [@Stuart2019]. 

# Installation

First of all, you need a current version of R (http://www.r-project.org). 
`r Biocpkg("progeny")` is a freely available annotation package deposited on 
http://bioconductor.org/ and https://https://github.com/saezlab/progeny.  

You can install it by running the following commands on an R console:

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("progeny")

## To install the new version until it is submitted to Bioconductor use:
devtools::install_github("saezlab/progeny")
```

We also load here the packages required to run this script. 

```{r, message=FALSE}
library(progeny)
library(dplyr)
library(Seurat)
library(ggplot2)
library(tidyr)
library(readr)
library(pheatmap)
library(tibble)
```

# Example of usage

In the following paragraphs, we provide examples describing how to run 
**PROGENy** on a scRNA-seq dataset. In particular, we use the 
`r CRANpkg("Seurat")` toolkit for single cell genomics [@Stuart2019]. For the 
sake of simplicity, we follow the example provided in the following 
`r CRANpkg("Seurat")` vignette:  

https://satijalab.org/seurat/v3.1/pbmc3k_tutorial.html 

The dataset contains 2,700 Peripheral Blood Mononuclear Cells (PBMC) that were 
sequenced on the Illumina NextSeq 500. This dataset is freely available in 
10X Genomics: 

https://s3-us-west-2.amazonaws.com/10x.files/samples/cell/pbmc3k/pbmc3k_filtered_gene_bc_matrices.tar.gz

```{r, eval=FALSE}
## Load the PBMC dataset
pbmc.data <- 
    Read10X(data.dir = "../data/pbmc3k/filtered_gene_bc_matrices/hg19/")
## Initialize the Seurat object with the raw (non-normalized data).
pbmc <- 
    CreateSeuratObject(counts = pbmc.data, project = "pbmc3k", min.cells = 3, 
    min.features = 200)
```

```{r, eval=TRUE , include=FALSE}
pbmc <- 
    readRDS(url("https://github.com/alberto-valdeolivas/ProgenyVignette/raw/master/SeuratObject.rds"))
```

## Pre-processing, normalization and identification of highly variable features

We follow the standard pre-processing steps as described in the aforementioned 
`r CRANpkg("Seurat")` vignette before going deeper into the data analysis. 
These steps carry out the selection and filtration of cells based on quality
control metrics, the data normalization and scaling, and the detection of highly
variable features (see https://satijalab.org/seurat/v3.1/pbmc3k_tutorial.html). 

```{r, message=FALSE}
## Identification of mithocondrial genes
pbmc[["percent.mt"]] <- PercentageFeatureSet(pbmc, pattern = "^MT-")

## Filtering cells following standard QC criteria.
pbmc <- subset(pbmc, subset = nFeature_RNA > 200 & nFeature_RNA < 2500 & 
    percent.mt < 5)

## Normalizing the data
pbmc <- NormalizeData(pbmc, normalization.method = "LogNormalize", 
    scale.factor = 10000)

pbmc <- NormalizeData(pbmc)

## Identify the 2000 most highly variable genes
pbmc <- FindVariableFeatures(pbmc, selection.method = "vst", nfeatures = 2000)

## In addition we scale the data
all.genes <- rownames(pbmc)
pbmc <- ScaleData(pbmc, features = all.genes)
```

## Clustering cells

One of the most relevant steps in scRNA-seq data analysis is clustering. Cells
are grouped based on the similarity of their transcriptomic profiles. We first 
apply the `r CRANpkg("Seurat")` v3 classical approach as described in their 
aforementioned vignette. We visualize the cell clusters using UMAP: 

```{r, message=FALSE, warning=FALSE}
pbmc <- 
    RunPCA(pbmc, features = VariableFeatures(object = pbmc), verbose = FALSE)
pbmc <- FindNeighbors(pbmc, dims = 1:10, verbose = FALSE)
pbmc <- FindClusters(pbmc, resolution = 0.5, verbose = FALSE)
pbmc <- RunUMAP(pbmc, dims = 1:10,  umap.method = 'uwot', metric='cosine')
```

```{r}
DimPlot(pbmc, reduction = "umap")
```

## Pathway activity per cell population

Following again `r CRANpkg("Seurat")` protocol, we next find the markers 
that help to identify and discriminate the different cell popualations 
present in the dataset under study.

```{r, message = FALSE}
## Finding differentially expressed features (cluster biomarkers)
pbmc.markers <- FindAllMarkers(pbmc, only.pos = TRUE, min.pct = 0.25, 
    logfc.threshold = 0.25, verbose = FALSE)

## Assigning cell type identity to clusters
new.cluster.ids <- c("Naive CD4 T", "Memory CD4 T", "CD14+ Mono", "B", "CD8 T", 
    "FCGR3A+ Mono", "NK", "DC", "Platelet")
names(new.cluster.ids) <- levels(pbmc)
pbmc <- RenameIdents(pbmc, new.cluster.ids)

## We create a data frame with the specification of the cells that belong to 
## each cluster to match with the Progeny scores.
CellsClusters <- data.frame(Cell = names(Idents(pbmc)), 
    CellType = as.character(Idents(pbmc)),
    stringsAsFactors = FALSE)
```

We plot again the clusters along with their cell type labels

```{r, message = FALSE, warning = FALSE}
DimPlot(pbmc, reduction = "umap", label = TRUE, pt.size = 0.5) + NoLegend()
```

Finally, we compute **PROGENy** pathway activity scores on the scRNA-seq data, 
and we then characterice the different cell populations based on these scores. 

```{r, message = FALSE}
## We compute the Progeny activity scores and add them to our Seurat object
## as a new assay called Progeny. 
pbmc <- progeny(pbmc, scale=FALSE, organism="Human", top=500, perm=1, 
    return_assay = TRUE)

## We can now directly apply Seurat functions in our Progeny scores. 
## For instance, we scale the pathway activity scores. 
pbmc <- Seurat::ScaleData(pbmc, assay = "progeny") 

## We transform Progeny scores into a data frame to better handling the results
progeny_scores_df <- 
    as.data.frame(t(GetAssayData(pbmc, slot = "scale.data", 
        assay = "progeny"))) %>%
    rownames_to_column("Cell") %>%
    gather(Pathway, Activity, -Cell) 

## We match Progeny scores with the cell clusters.
progeny_scores_df <- inner_join(progeny_scores_df, CellsClusters)

## We summarize the Progeny scores by cellpopulation
summarized_progeny_scores <- progeny_scores_df %>% 
    group_by(Pathway, CellType) %>%
    summarise(avg = mean(Activity), std = sd(Activity))
```

We plot the different pathway activities for the different cell populations

```{r, message=FALSE}
## We prepare the data for the plot
summarized_progeny_scores_df <- summarized_progeny_scores %>%
    dplyr::select(-std) %>%   
    spread(Pathway, avg) %>%
    data.frame(row.names = 1, check.names = FALSE, stringsAsFactors = FALSE) 
```

```{r}
paletteLength = 100
myColor = colorRampPalette(c("Darkblue", "white","red"))(paletteLength)

progenyBreaks = c(seq(min(summarized_progeny_scores_df), 0, 
                      length.out=ceiling(paletteLength/2) + 1),
                  seq(max(summarized_progeny_scores_df)/paletteLength, 
                      max(summarized_progeny_scores_df), 
                      length.out=floor(paletteLength/2)))
progeny_hmap = pheatmap(t(summarized_progeny_scores_df[,-1]),fontsize=14, 
                        fontsize_row = 10, 
                        color=myColor, breaks = progenyBreaks, 
                        main = "PROGENy (500)", angle_col = 45,
                        treeheight_col = 0,  border_color = NA)
```

# Session info {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progenySuppFunc.r
\name{progenyScatter}
\alias{progenyScatter}
\title{Calculate Progeny Scores with Permutations}
\usage{
progenyScatter(
  df,
  weight_matrix,
  dfID = 1,
  weightID = 1,
  statName = "gene stats",
  verbose = FALSE
)
}
\arguments{
\item{df}{an n*m data frame, where n is the number of omic features (genes). 
m isn't really important, as long as at least one column corresponds to a 
sample or contrast statistic. One of the columns should correspond to the gene 
symbols.}

\item{weight_matrix}{A progeny coefficient matrix. the first column should be
the identifiers of the omic features, and should be coherent with 
the identifiers provided in df.}

\item{dfID}{an integer corresponding to the column number of 
the gene identifiers of df.}

\item{weightID}{an integer corresponding to the column number of the gene 
identifiers of the weight matrix.}

\item{statName}{The name of the stat used, to be displayed on the plot}

\item{verbose}{Logical indicating whether we want to have the messages 
indicating the different computed weights.}
}
\value{
The function returns a list of list of arrangeGrob objects.
The first level list elements correspond to samples/contrasts. 
The second level correspond to pathways.
The plots can be saved in a pdf format using the saveProgenyPlots function.
}
\description{
This function generate a series of scatter plot with marginal distribution
(in the form of an arrangeGrob object), for each progeny pathway and
sample/contrast. Each scatter plot has progeny weights as x-axis and the gene
level stat used to compute progeny score as the y-axis. The marginal
distribution of the gene level stats is displayed on the right of the plot
to give visual support of the significance of each gene contributing to
the progeny pathway score. The green and red colors represent the positive
and negative contribution of genes to the progeny pathway, respectively.
For each gene contribution, 4 cases are possible, as the combinations of
the sign of the gene level stat and the sign of the gene level weight.
Positive weight will lead to a positive(green)/negative(red) gene contribution
if the gene level stat is positive/negative. Negative weight will lead to
a negative(red)/positive(green) gene contribution if the gene level stat
is positive/negative.
}
\examples{
# use example gene expression matrix

gene_expression <- read.csv(system.file("extdata", 
"human_input.csv", package = "progeny"))

# getting a model matrix with 100 top significant genes and converting to df
weight_matrix <- getModel("Human", top=100)
weight_matrix <- data.frame(names = row.names(weight_matrix), 
  row.names = NULL, weight_matrix)

#use progenyScatter function
plots <- progenyScatter(gene_expression, weight_matrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\name{vignette_data}
\alias{vignette_data}
\title{The RNA data used in the progeny vignette}
\format{
List with three elements: the gene counts, the experimental design and 
the result of limma differential analysis        
\describe{
    \item{counts}{gene counts}
    \item{design}{experiemental design}
    \item{limma_ttop}{differential analysis result using limma}
}
}
\source{
\url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE119931}
}
\description{
List with three elements: the gene counts, the experimental design and 
the result of limma differential analysis
}
\examples{
data("vignette_data")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progenySuppFunc.r
\name{saveProgenyPlots}
\alias{saveProgenyPlots}
\title{Function to save Progeny plots}
\usage{
saveProgenyPlots(plots, contrast_names, dirpath)
}
\arguments{
\item{plots}{a list of list of arrangeGrob object (such as the one returned 
by the progenyScatter function.).The first level list elements correspond 
to samples/contrasts. The second level corresponds to pathways.
The plots can be saved in a pdf format using the saveProgenyPlots function.}

\item{contrast_names}{a vector of the same length as the first level of 
the plot list corresponding to the names of each sample/contrast}

\item{dirpath}{the path to the directory where the plots should be saved}
}
\value{
This function produces the pdf files of plots taken from the 
progenyScatter function
}
\description{
This function is designed to save the plots (in pdf format) of a nested 
(2 level) list of arrangeGrob objects, such as the one returned by 
the progenyScatter function.
}
\examples{
#create plots using progneyScatter function
gene_expression <- read.csv(system.file("extdata", 
"human_input.csv", package = "progeny"))

# getting a weight_matrix
weight_matrix <- getModel("Human", top=100)
weight_matrix <- data.frame(names = row.names(weight_matrix), 
  row.names = NULL, weight_matrix) 
plots <- progenyScatter(gene_expression, weight_matrix)

#create a list with contrast names
contrast_names <- names(gene_expression[2:ncol(gene_expression)])

#assign a path to store your plots
dirpath <- "./progeny_plots/"

# save it
# saveProgenyPlots(plots, contrast_names, dirpath)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progeny-package.r, R/progeny.r
\docType{package}
\name{progeny}
\alias{progeny}
\title{Calculate PROGENy pathway scores from gene expression}
\usage{
progeny(
  expr,
  scale = TRUE,
  organism = "Human",
  top = 100,
  perm = 1,
  verbose = FALSE,
  z_scores = FALSE,
  get_nulldist = FALSE,
  assay_name = "RNA",
  return_assay = FALSE,
  ...
)
}
\arguments{
\item{expr}{A gene expression object with HGNC/MGI symbols in rows and 
samples in columns. In order to run PROGENy in single-cell 
RNAseq data, it also accepts Seurat and SingleCellExperiment 
object, taking the normalized counts for the computation.}

\item{scale}{A logical value indicating whether to scale the scores of each
pathway to have a mean of zero and a standard deviation of one.
It does not apply if we use permutations.}

\item{organism}{The model organism - "Human" or "Mouse"}

\item{top}{The top n genes for generating the model matrix according to
significance (p-value)}

\item{perm}{An interger detailing the number of permutations. No 
permutations by default (1). When Permutations larger than 1,
we compute progeny pathway scores and assesses their 
significance using a gene sampling-based permutation strategy, 
for a series of experimental samples/contrasts.}

\item{verbose}{A logical value indicating whether to display a message
about the number of genes used per pathway to compute 
progeny scores (i.e. number of genes present in the 
progeny model and in the expression dataset)}

\item{z_scores}{Only applies if the number of permutations is greater than 1. 
A logical value. TRUE: the z-scores will be returned for 
the pathway activity estimations. FALSE: the function returns 
a normalized z-score value between -1 and 1.}

\item{get_nulldist}{Only applies if the number of permutations is greater 
than 1. A logical value. TRUE: the null distributions
generated to assess the signifance of the pathways scores 
is also returned.}

\item{assay_name}{Only applies if the input is a Seurat object. It selects the
name of the assay on which Progeny will be run. Default to: 
RNA, i.e. normalized expression values.}

\item{return_assay}{Only applies if the input is a Seurat object. A logical 
value indicating whether to return progeny results as a new 
assay called Progeny in the Seurat object used as input. 
Default to FALSE.}

\item{...}{Additional arguments to be passed to the functions.}
}
\value{
A matrix with samples in rows and pathways in columns. In case
              we run the method with permutations and the option get_nulldist
              to TRUE, we will get a list with two elements. The first 
              element is the matrix with the pathway activity as before. 
              The second elements is the null distributions that we generate
              to assess the signifance of the pathways scores.
}
\description{
This function uses the linear model of pathway-responsive genes underlying
the PROGENy method. It transforms a gene expression matrix with HGNC/MGI gene
symbols in rows and sample names in columns into a pathway score matrix with
samples in rows and pathways in columns.
}
\details{
The publication of the method is available at:
https://www.nature.com/articles/s41467-017-02391-6

The supplied expression object has to contain HGNC/MGI symbols in rows. This
will, in most cases (and how we originally used it), be either normalized
gene expression of a microarray experiment or log-transformed (and
possible variance-stabilized) counts from an RNA-seq experiment.

The human and mouse model matrices consists of 14 pathways and large set of 
genes with an associated p-value (p-value per gene and pathway) that accounts
for the importance of each gene on each pathway upon perturbation. 
Its coefficients are non-zero if the gene-pathway pair corresponds
to the top N genes (100 by default) that were up-regulated upon stimulation
of the pathway in a wide range of experiments. The value corresponds to the 
fitted z-score across experiments in our model fit. 
Only rows with at least one non-zero coefficient were included, as the rest 
is not used to infer pathway activity.
}
\examples{
# use example gene expression matrix here, this is just for illustration
gene_expression <- as.matrix(read.csv(system.file("extdata", 
"human_input.csv", package = "progeny"), row.names = 1))

# calculate pathway activities
pathways <- progeny(gene_expression, scale=TRUE, 
    organism="Human", top = 100, perm = 1)
}
\seealso{
\code{\link{progenyPerm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progenyPermutations.r
\name{progenyPerm}
\alias{progenyPerm}
\title{Compute progeny pathway scores and assesses significance based on permutations}
\usage{
progenyPerm(
  df,
  weight_matrix,
  k = 10000,
  z_scores = TRUE,
  get_nulldist = FALSE
)
}
\arguments{
\item{df}{A data.frame of n*m+1 dimension, where n is the number of omic
features to be considered and m is the number of samples/contrasts.
The first column should be the identifiers of the omic features. 
These identifiers must be coherent with the identifiers of the weight matrix.}

\item{weight_matrix}{A progeny coefficient matrix. the first column should be
the identifiers of the omic features and should be coherent with 
the identifiers provided in df.}

\item{k}{The number of permutations to be performed to generate
the null-distribution used to estimate the significance of progeny scores.  
The default value is 10000.}

\item{z_scores}{if true, the z-scores will be returned for 
the pathway activity estimations. Else, the function returns 
a normalized z-score value between -1 and 1.}

\item{get_nulldist}{if true, the null score distribution used for 
normalization will be returned along with the actual normalized score data 
frame.}
}
\value{
This function returns a list of two elements. The first element is 
a data frame of p*m+1 dimensions, where p is the number of progeny pathways,
and m is the number of samples/contrasts. Each cell represents the 
significance of a progeny pathway score for one sample/contrast. The 
significance ranges between -1 and 1. The significance is equal to x*2-1, x 
being the quantile of the progeny pathway score with respect to the null 
distribution. Thus, this significance can be interpreted as the equivalent of 
1-p.value two-sided test over an empirical distribution) with the sign 
indicating the direction of the regulation. The second element is the null 
distribution list (a null distribution is generated for each sample/contrast).
}
\description{
Compute progeny pathway scores and assesses significance based on permutations
}
\examples{
# use example gene expression matrix
gene_expression <- as.matrix(read.csv(system.file("extdata", 
"human_input.csv", package = "progeny"), row.names = 1))

# calculate pathway activities
progeny(gene_expression, scale=TRUE, organism="Human", top=100, perm=10000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/progenySuppFunc.r
\name{getModel}
\alias{getModel}
\title{Returns the Progeny Model}
\usage{
getModel(organism = "Human", top = 100, decoupleR = F)
}
\arguments{
\item{organism}{"Human" or "Mouse" taken from the main function's argument. 
Default to "Human"}

\item{top}{Desired top number of genes for each pathway according to their
significance(p.value). Default to 100}

\item{decoupleR}{if TRUE, the model matrix is goign to be generated with a format 
makes it directlz compatible with the decoupleR package}
}
\value{
This function returns model matrix according to the top n significant
}
\description{
This function is designed for getting a model matrix with top significant
genes for each pathway
}
\examples{
#getting a model matrix according to the desired top n significant 
model <- getModel("Human", top=100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\name{model_human_full}
\alias{model_human_full}
\title{The full human linear model underlying PROGENy}
\format{
The full human model contains 22479 genes, associated pathways,
weight and the p-value.         
\describe{
    \item{gene}{gene names in HGNC symbols}
    \item{pathway}{names of PROGENy pathways}
    \item{weight}{z-scores for a given gene}
    \item{p.value}{significance of gene in pathway}
}
}
\source{
\url{https://www.nature.com/articles/s41467-017-02391-6}
}
\description{
HGNC gene symbols in rows, pathways in columns. Pathway activity inference
works by matrix multiplication of gene expression with the model.
}
\examples{
get("model_human_full", envir = .GlobalEnv)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\name{model_mouse_full}
\alias{model_mouse_full}
\title{The full mouse linear model underlying PROGENy}
\format{
The full mouse model contains 17426 genes, associated pathways,
weight and the p-value.         
\describe{
    \item{gene}{gene names in HGNC symbols}
    \item{pathway}{names of PROGENy pathways}
    \item{weight}{z-scores for a given gene}
    \item{p.value}{significance of gene in a pathway}
}
}
\source{
\url{https://www.ncbi.nlm.nih.gov/pubmed/31525460}
}
\description{
MGI gene symbols in rows, pathways in columns. Pathway activity inference
works by matrix multiplication of gene expression with the model.
}
\examples{
get("model_mouse_full", envir = .GlobalEnv)
}
\keyword{datasets}
